@@ -77,12 +77,8 @@ class TestLoginWithPassword(object):
         self.request.cookies = self.saved_cookie
         self.request.user = self.saved_user
 
-        # Remove user name to id cache, or next test will fail
-        caching.CacheEntry(self.request, 'user', 'name2id', scope='wiki').remove()
-        try:
-            del self.request.cfg.cache.name2id
-        except:
-            pass
+        # Remove user lookup caches, or next test will fail
+        user.clearLookupCaches(self.request)
 
     def testAsciiPassword(self):
         """ user: login with ascii password """
@@ -245,8 +241,8 @@ class TestLoginWithPassword(object):
         assert not theUser.isSubscribedTo([testPagename]) # list(!) of pages to check
 
     def testRenameUser(self):
-        """ create user and then rename user and check
-        if the old username is removed from the cache name2id
+        """ create user and then rename user and check whether
+        the old username is removed (and the lookup cache behaves well)
         """
         # Create test user
         name = u'__Some Name__'
