@@ -12,9 +12,9 @@
 """
 
 import re
+import difflib
 
 from MoinMoin import config, wikiutil
-from MoinMoin.support import difflib
 from MoinMoin.Page import Page
 
 
