@@ -28,6 +28,7 @@
 """
 
 from datetime import datetime
+import hmac
 
 from MoinMoin import log
 logging = log.getLogger(__name__)
@@ -39,7 +40,6 @@ import mimetypes
 from MoinMoin import config, caching
 from MoinMoin.util import filesys
 from MoinMoin.action import AttachFile
-from MoinMoin.support.python_compatibility import hmac_new
 
 action_name = __name__.split('.')[-1]
 
@@ -99,7 +99,7 @@ def key(request, wikiname=None, itemname=None, attachname=None, content=None, se
         raise AssertionError('cache_key called with unsupported parameters')
 
     hmac_data = hmac_data.encode('utf-8')
-    key = hmac_new(secret, hmac_data).hexdigest()
+    key = hmac.new(secret, hmac_data).hexdigest()
     return key
 
 
