@@ -2,10 +2,10 @@
 """
     MoinMoin - image/jpeg Filter
 
-    @copyright: 2006 by ThomasWaldmann MoinMoin:ThomasWaldmann
+    @copyright: 2006 MoinMoin:ThomasWaldmann
     @license: GNU GPL, see COPYING for details.
 """
-import EXIF
+from MoinMoin.filter import EXIF
 
 def execute(indexobj, filename):
     """ Extract some EXIF data """
@@ -23,8 +23,9 @@ def execute(indexobj, filename):
         except:
             pass
         data = str(tags).decode('utf-8')
-    except (ValueError, TypeError), err: # EXIF throws ValueError on unknown tags
-                                         #             TypeError on other occassions
+    except (ValueError, TypeError, KeyError): # EXIF throws ValueError on unknown tags
+                                              # TypeError on other occassions
+                                              # KeyError too
         data = u''
     return data
 
