@@ -76,10 +76,13 @@ class EventLog:
 
     def read(self):
         """ read complete event-log from disk """
+        print "Reading event-log. Depending on the size of event-log this may take a long time and"
+        print "consume lots of memory (if you don't need statistics, truncate event-log to speed this"
+        print "up and use less memory)."
         data = []
         try:
             lineno = 0
-            f = file(self.fname, 'r')
+            f = file(self.fname, 'r') # read in text mode, so we can iterate over text lines
             for line in f:
                 lineno += 1
                 line = line.replace('\r', '').replace('\n', '')
@@ -102,8 +105,10 @@ class EventLog:
 
     def write(self, fname):
         """ write complete event-log to disk """
+        print "Writing event-log. Depending on the size of event-log this may take a long time."
         if self.data:
-            f = file(fname, 'w')
+            f = file(fname, 'wb') # write in binary mode, so it stays exactly as we write it, even on windows.
+                                  # the code in MoinMoin.logfile also uses binary mode and writes \n only.
             for timestamp, action, kvdict in self.data:
                 pagename = kvdict.get('pagename')
                 if pagename and ('PAGE', pagename) in self.renames:
@@ -132,7 +137,7 @@ class EditLog:
         data = {}
         try:
             lineno = 0
-            f = file(self.fname, 'r')
+            f = file(self.fname, 'r') # read in text mode, so we can iterate over text lines
             for line in f:
                 lineno += 1
                 line = line.replace('\r', '').replace('\n', '')
@@ -160,7 +165,8 @@ class EditLog:
         if self.data:
             editlog = self.data.items()
             editlog.sort()
-            f = file(fname, "w")
+            f = file(fname, 'wb') # write in binary mode, so it stays exactly as we write it, even on windows.
+                                  # the code in MoinMoin.logfile also uses binary mode and writes \n only.
             max_rev = 0
             for key, fields in editlog:
                 timestamp, rev, action, pagename, ip, hostname, userid, extra, comment = fields
