@@ -5,19 +5,21 @@
     a sub-command.
 
     Usage: moin cmdmodule cmdname [options]
-               
-    @copyright: 2006 by Thomas Waldmann
+
+    @copyright: 2006 MoinMoin:ThomasWaldmann
     @license: GNU GPL, see COPYING for details.
 """
 
 def run():
-    from MoinMoin.script import _util
-    _util.MoinScript().run(showtime=0)
-    
+    from MoinMoin.script import MoinScript
+    MoinScript().run(showtime=0)
+
 if __name__ == "__main__":
     # Insert the path to MoinMoin in the start of the path
     import sys, os
-    sys.path.insert(0, os.path.join(os.path.dirname(sys.argv[0]), os.pardir, os.pardir))
+    # we use position 1 (not 0) to give a config dir inserted at 0 a chance
+    # beware: we have a wikiconfig.py at the toplevel directory in the branch
+    sys.path.insert(1, os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), os.pardir, os.pardir)))
 
     run()
 
