@@ -16,10 +16,11 @@ from MoinMoin import security
 acliter = security.ACLStringIterator
 AccessControlList = security.AccessControlList
 
+from MoinMoin.datastruct import ConfigGroups
 from MoinMoin.PageEditor import PageEditor
 from MoinMoin.user import User
 
-from MoinMoin._tests import become_trusted, create_page, nuke_page
+from MoinMoin._tests import wikiconfig, become_trusted, create_page, nuke_page
 
 class TestACLStringIterator(object):
 
@@ -248,6 +249,50 @@ class TestAcl(object):
                 assert not acl.may(self.request, user, right)
 
 
+class TestGroupACL(object):
+
+    class Config(wikiconfig.Config):
+        def groups(self, request):
+            groups = {
+                u'PGroup': frozenset([u'Antony', u'Beatrice', ]),
+                u'AGroup': frozenset([u'All', ]),
+                # note: the next line is a INTENDED misnomer, there is "All" in
+                # the group NAME, but not in the group members. This makes
+                # sure that a bug that erroneously checked "in groupname" (instead
+                # of "in groupmembers") does not reappear.
+                u'AllGroup': frozenset([]), # note: intended misnomer
+            }
+            return ConfigGroups(request, groups)
+
+    def testApplyACLByGroup(self):
+        """ security: applying acl by group name"""
+        # This acl string...
+        acl_rights = [
+            "PGroup,AllGroup:read,write,admin "
+            "AGroup:read "
+            ]
+        acl = security.AccessControlList(self.request.cfg, acl_rights)
+
+        # Should apply these rights:
+        users = (
+            # user, rights
+            ('Antony', ('read', 'write', 'admin', )),  # in PGroup
+            ('Beatrice', ('read', 'write', 'admin', )),  # in PGroup
+            ('Charles', ('read', )),  # virtually in AGroup
+            )
+
+        # Check rights
+        for user, may in users:
+            mayNot = [right for right in self.request.cfg.acl_rights_valid
+                      if right not in may]
+            # User should have these rights...
+            for right in may:
+                assert acl.may(self.request, user, right)
+            # But NOT these:
+            for right in mayNot:
+                assert not acl.may(self.request, user, right)
+
+
 class TestPageAcls(object):
     """ security: real-life access control list on pages testing
     """
