@@ -1,36 +1,60 @@
 #!/usr/bin/env python
+# -*- coding: iso-8859-1 -*-
 """
 MoinMoin wiki project -> Google Project Hosting converter
 
-full of evil antipatterns, incl. Exception exceptions
-
-@copyright: 2007 MoinMoin:AlexanderSchremmer
-@license: GPL v2
+Full of evil antipatterns, incl. Exception exceptions.
 
+@copyright: 2007,2010 MoinMoin:AlexanderSchremmer
+@license: GNU GPL, see COPYING for details.
 """
+
 import sys
 import re
 import urllib2
 from urllib import quote
 import xmlrpclib
+import csv
 
-sys.path.append("/home/alexander/dev/python/ghopimport")
-#sys.path.append("/srv/moin_tw/moin-1.6/contrib")
+from MoinMoin.web.contexts import ScriptContext
+from MoinMoin.Page import Page
 
-from googleimport import googlepush
+# monkeypatch the formatter to avoid line_anchors:
+from MoinMoin.formatter import text_html
+text_html.line_anchors = False
+
+request = ScriptContext(None, None)
 
 
 class DataNotFoundException(Exception): pass
 
 
 class Task(object):
-    def __init__(self, summary, desc, label):
+    def __init__(self, summary, desc, label, hours, mentors, difficulty, types):
         self.summary = summary
-        self.desc = desc
         self.label = label
+        self.hours = hours
+        self.mentors = mentors
+        self.difficulty = difficulty
+        self.types = types
+
+        page = Page(request, "")
+        page.set_raw_body(desc)
+        desc = request.redirectedOutput(page.send_page, content_only=1)
+        for s, r in [
+                ('\n', ' '),
+                (' class="line862"', ''),
+                (' class="line867"', ''),
+                (' class="line874"', ''),
+                (' class="line891"', ''),
+            ]:
+            desc = desc.replace(s, r)
+        self.desc = desc
 
     def __repr__(self):
-        return (u"<Task summary=%r label=%r desc='''%s'''>" % (self.summary, self.label,  self.desc[:100] )).encode("utf-8")
+        return (u"<Task summary=%r label=%r hours=%i mentors=%r difficulty=%r types=%r desc='''%s'''>" % (
+            self.summary, self.label, self.hours, self.mentors, self.difficulty,
+            self.types, self.desc[:100])).encode("utf-8")
 
 
 def find_dict_entry(name, text):
@@ -39,70 +63,30 @@ def find_dict_entry(name, text):
         raise DataNotFoundException("%s not found" % (name, ))
     return m.groups()[0]
 
-desc_pattern = r"""== Short Description ==
+
+desc_pattern = r"""= Description =
 ([\s\S]*?)
-(= |$)"""
+= Discussion ="""
 
 bugpage_pattern = r"""= Description =
 ([\s\S]*?)
 ="""
 
 already_pushed_pages = set([x.strip() for x in """
-EasyToDo/ExtendFormsOfAdvancedSearch
-EasyToDo/ResearchMacOSXPluginSupport
-EasyToDo/Research_Python_code_usable_for_filters
-EasyToDo/Code_vCard_hCard_Support_For_Wikihomepages
-EasyToDo/UserPreferredLanguageStatistics
-EasyToDo/CloneWikiPagesByPackagePages
-EasyToDo/ResearchLinuxPluginSupport
-EasyToDo/ConvertMacrosToNewSyntax
-EasyToDo/CaseStudy
-EasyToDo/InstallMoinMoinForYourFriends
-EasyToDo/JabberBotRefactoring
-EasyToDo/ProofreadEnglishDocumentation
-EasyToDo/ImplementXEPEntityCapabilities 1
-EasyToDo/ImplementXEPEntityCapabilities 3
-EasyToDo/ImplementXEPEntityCapabilities 2
-EasyToDo/TestInstallDocs
-EasyToDo/DesignAMoinMoinTheme
-EasyToDo/ImproveStyleOfModernTheme
-EasyToDo/ThinkingAloudUsabilityTest
-EasyToDo/MakeAScreencast
-EasyToDo/IntroduceMoinMoinToYourFriends
-EasyToDo/RunJabberBotOnWindows
-EasyToDo/ResearchWindowsPluginSupport
-EasyToDo/DesignNewIconset
-EasyToDo/CreateAPoster
-EasyToDo/GermanWikiKurs
-EasyToDo/Firefox3CompatibilityCheck
-EasyToDo/SearchForMoinMoinIntegration
-EasyToDo/AddUsageInfoToMoinCommand
-EasyToDo/DumpPagesIntoZip
-EasyToDo/ShowAclIndicator
 """.split("\n")])
 
 already_pushed_bugs = set([x.strip() for x in """
-1.6devFAT32TroubleWithUnderlayFileNames
-1.6devMissingRightsI18n
-AclBlockMoinDump
-ArbitraryInjectionOfErrorMessage
-CannotUpdateCreateDrawings
-GuiEditorExcelPasteExpatErrorUnboundPrefix
-MailAccountDataGivesError
-MakeIconLinkLosesAltTitle
-ModPyConnectionErrors
-MoinDumpTheme
-NavigationMacroMultipleRepeat
-RenamingUserAllowsOldUsernameToLogin
-SubscribeAndUnsubscribeShareSameUrl
-TWikiDrawOnDebian
-TrivialChangeEasyAccess
-WrongAlignedAttachment
-XmlRpcPutPageAllowsEmptyPageName
 """.split("\n")])
 
 gatherers = []
 
+def first(s):
+    """ return first word or '' """
+    splitted = s.strip().split()
+    if splitted:
+        return splitted[0]
+    else:
+        return ''
 
 class Collector(object):
     def is_gatherer(function):
@@ -129,27 +113,35 @@ class Collector(object):
                 continue
             page_contents = self.server.getPage(page)
             try:
-                summary = find_dict_entry("Summary", page_contents)
-                count = int(find_dict_entry("Count", page_contents))
-                label = find_dict_entry("Label", page_contents)
-            except DataNotFoundException, e:
-                print "Could not import %r because of %r" % (page, e)
+                summary = find_dict_entry("Title", page_contents)
+                count = int(first(find_dict_entry("Count", page_contents)))
+                label = find_dict_entry("Tags", page_contents)
+                hours = int(first(find_dict_entry("Duration", page_contents)))
+                mentors = find_dict_entry("Mentors", page_contents)
+                difficulty = find_dict_entry("Difficulty", page_contents)
+                try:
+                    types = find_dict_entry("Types", page_contents)
+                except DataNotFoundException:
+                    # old tasks use "Type"
+                    types = find_dict_entry("Type", page_contents)
+            except (DataNotFoundException, ValueError), e:
+                print >>sys.stderr, "Could not import %r because of %r" % (page, e)
                 continue
             desc_m = re.search(desc_pattern, page_contents)
             if not desc_m:
-                raise Exception("Desc not found")
+                raise Exception("Could not import %r because Desc not found" % page)
             desc = desc_m.groups()[0]
+
             for i in range(1, count + 1):
-                print page
                 text = desc
                 new_summary = summary
-                text += "\n\nA more detailed description of this issue is available at the MoinMoin wiki: %s" % (self.url + quote(page.encode("utf-8")), )
+                text += "\n\nYou can discuss this issue in the !MoinMoin wiki: %s" % (self.url + quote(page.encode("utf-8")), )
                 if count > 1:
                     text += "\n\nThis issue is available multiple times. This one is %i of %i." % (i, count)
                     new_summary += " %i/%i" % (i, count)
-                yield Task(new_summary, text, label)
+                yield Task(new_summary, text, label, hours, mentors, difficulty, types)
 
-    @is_gatherer
+    #@is_gatherer
     def moin_bugs(self):
         pages = [pagename for pagename, contents in self.server.searchPages(r"t:MoinMoinBugs/ r:CategoryEasy\b")]
         for page in pages:
@@ -159,7 +151,7 @@ class Collector(object):
             page_contents = self.server.getPage(page)
             m = re.search(bugpage_pattern, page_contents)
             if not m:
-                raise Exception("bug desc not found")
+                raise Exception("Could not import %r because of bug desc not found" % page)
             desc = m.groups()[0]
             desc = "A user filed a bug report at the MoinMoin site. Here is a short description about the issue. A more detailed description is available at the MoinMoin wiki: %s\n\n" % (self.url + quote(page.encode("utf-8")), ) + desc
             yield Task(bug_name, desc, "Code")
@@ -174,10 +166,10 @@ class Collector(object):
             page_contents = self.server.getPage(page)
             page_contents = page_contents.replace("LANG", language)
             summary = find_dict_entry("Summary", page_contents)
-            count = int(find_dict_entry("Count", page_contents))
+            count = int(first(find_dict_entry("Count", page_contents)))
             desc_m = re.search(desc_pattern, page_contents)
             if not desc_m:
-                raise Exception("Desc not found")
+                raise Exception("Could not import %r because Desc not found" % page)
             desc = desc_m.groups()[0]
             for i in range(1, count + 1):
                 text = desc
@@ -189,40 +181,19 @@ class Collector(object):
                 yield Task(new_summary, text, "Translation")
 
 
-def pull_and_push():
-    #project_name = "google-highly-open-participation-moinmoin" # PRODUCTION IMPORT
-    project_name = "moin-sandbox" # TEST RUN
-    summary_prefix = "" # EMPTY FOR PRODUCTION IMPORT!
-    if summary_prefix:
-        tmin, tmax = 0, None
-    else:
-        tmin, tmax = 0, None
-    print "Collecting tasks ..."
+def pull_and_gencsv():
+    print >> sys.stderr, "Collecting tasks ..."
     tasks = Collector("http://moinmo.in/").collect_tasks()
-    print "Importing %i tasks ..." % (len(tasks), )
-    print "\n".join(repr(task) for task in tasks)
-    argc = len(sys.argv)
-    if not (2 <= argc <= 3):
-        raise SystemExit("you must supply your username (and optionally your password) as argument(s) to this program")
-    user = sys.argv[1]
-    if argc == 2:
-        password = raw_input("Password for %s:" % user)
-    else:
-        password = sys.argv[2]
+    print >> sys.stderr, "Importing %i tasks ..." % (len(tasks), )
+    print >> sys.stderr, "\n".join(repr(task) for task in tasks)
 
-
-    try:
-        googlepush.login(user, password)
-    except urllib2.URLError, e:
-        print "Ignored exception %r" % (e, )
-
-    i = 0
+    summary_prefix = '' # "[TEST] " # EMPTY FOR PRODUCTION IMPORT!
+    tmin, tmax = 0, None
+    csvwriter = csv.writer(sys.stdout, delimiter=",", doublequote=True)
     for task in tasks[tmin:tmax]:
-        i += 1
-        print i, repr(task.summary)
-        googlepush.push_item(project_name, summary_prefix + task.summary, task.desc, "Open", task.label)
+        csvwriter.writerow([summary_prefix + task.summary, task.desc, task.hours, task.mentors, task.difficulty, task.types, task.label])
 
 
 if __name__ == "__main__":
-    pull_and_push()
+    pull_and_gencsv()
 
